# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from abc import abstractmethod
from dataclasses import dataclass
from enum import Enum, auto
from typing import List
from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.QtNetwork import QTcpSocket, QAbstractSocket
from typing import Optional

from LANDrop.crypto import Crypto


class State(Enum):
    HANDSHAKE1 = auto()
    HANDSHAKE2 = auto()
    TRANSFERRING = auto()
    FINISHED = auto()


class FileTransferSession(QObject):
    printMessage = pyqtSignal(str)
    updateProgress = pyqtSignal(float)
    errorOccurred = pyqtSignal(str)
    fileMetadataReady = pyqtSignal(list, int, str, str)
    ended = pyqtSignal()

    @dataclass
    class FileMetadata:
        filename: str
        size: int

    def __init__(self, parent: Optional['QObject'], socket: QTcpSocket) -> None:
        super().__init__(parent)
        self.state = State.HANDSHAKE1
        self.socket: QTcpSocket = socket
        self.totalSize = 0
        self.transferredSize = 0
        self.crypto = Crypto()
        self.readBuffer = b""
        self.transferQ: List[FileTransferSession.FileMetadata] = []

        self.socket.setParent(self)
        self.socket.setSocketOption(QAbstractSocket.LowDelayOption, 1)
        self.socket.readyRead.connect(self.socketReadyRead)
        self.socket.error.connect(self.socketErrorOccurred)

    def start(self):
        self.printMessage.emit(self.tr("Handshaking..."))
        self.socket.write(self.crypto.localPublicKey())

    @abstractmethod
    def respond(self, accepted: bool):
        raise RuntimeError("respond not implemented")

    def encryptAndSend(self, data: bytes) -> None:
        sendData: bytes = self.crypto.encrypt(data)
        size = len(sendData)
        sendData = bytes([size & 0xFF]) + sendData
        sendData = bytes([(size >> 8) & 0xFF]) + sendData
        self.socket.write(sendData)

    @abstractmethod
    def handshake1Finished(self) -> None:
        pass

    @abstractmethod
    def processReceivedData(self, data: bytes) -> None:
        pass

    def socketReadyRead(self) -> None:

        self.readBuffer += bytes(self.socket.readAll())
        if self.state == State.HANDSHAKE1:
            if len(self.readBuffer) < self.crypto.publicKeySize():
                self.errorOccurred.emit(self.tr("Handshake failed."))
                return
            publicKey = self.readBuffer[:self.crypto.publicKeySize()]
            self.readBuffer = self.readBuffer[self.crypto.publicKeySize():]
            try:
                self.crypto.setRemotePublicKey(publicKey)
            except Exception as e:
                self.errorOccurred.emit(str(e))
                return
            self.printMessage.emit(self.tr("Handshaking... Code: %1"
                                           ).replace(
                "%1", self.crypto.sessionKeyDigest()
            ))
            self.state = State.HANDSHAKE2

            self.handshake1Finished()

        while self.readBuffer:
            if len(self.readBuffer) < 2:
                break

            size = self.readBuffer[0] << 8
            size |= self.readBuffer[1]
            if len(self.readBuffer) < size + 2:
                break

            data = self.readBuffer[2: size+2]
            self.readBuffer = self.readBuffer[size + 2:]

            try:
                data = self.crypto.decrypt(data)
            except RuntimeError as e:
                self.errorOccurred.emit(str(e))
                return

            self.processReceivedData(data)

    def socketErrorOccurred(self) -> None:
        if self.state != State.FINISHED:
            self.errorOccurred.emit(self.socket.errorString())



from nacl.bindings.randombytes import randombytes
from nacl.bindings.crypto_scalarmult import crypto_scalarmult_SCALARBYTES, crypto_scalarmult_base, crypto_scalarmult
from nacl.bindings.crypto_aead import crypto_aead_chacha20poly1305_ietf_KEYBYTES, \
    crypto_aead_chacha20poly1305_ietf_encrypt, crypto_aead_chacha20poly1305_ietf_decrypt, \
    crypto_aead_chacha20poly1305_ietf_NPUBBYTES
from nacl.bindings.crypto_generichash import crypto_generichash_BYTES, crypto_generichash_BYTES_MIN, \
    crypto_generichash_BYTES_MAX, crypto_generichash_KEYBYTES_MAX
from nacl.bindings.crypto_generichash import generichash_blake2b_init, generichash_blake2b_update, \
    generichash_blake2b_final


def generichash_blake2b(data: bytes,
                        digest_size: int = crypto_generichash_BYTES,
                        key: bytes = b""
                        ):
    state = generichash_blake2b_init(key=key, digest_size=digest_size)
    generichash_blake2b_update(state, data)
    return generichash_blake2b_final(state)


def crypto_generichash(data: bytes,
                       digest_size: int = crypto_generichash_BYTES,
                       key: bytes = b""):
    return generichash_blake2b(data, digest_size, key)


__all__ = [
    "randombytes",
    "crypto_generichash",
    "crypto_scalarmult_SCALARBYTES",
    "crypto_scalarmult_base",
    "crypto_scalarmult",
    "crypto_aead_chacha20poly1305_ietf_KEYBYTES",
    "crypto_aead_chacha20poly1305_ietf_encrypt",
    "crypto_aead_chacha20poly1305_ietf_decrypt",
    "crypto_generichash_BYTES",
    "crypto_generichash_BYTES_MIN",
    "crypto_generichash_BYTES_MAX",
    "crypto_generichash_KEYBYTES_MAX",
    "crypto_aead_chacha20poly1305_ietf_NPUBBYTES",
]

if __name__ == "__main__":
    skey = randombytes(crypto_scalarmult_SCALARBYTES)
    pkey = crypto_scalarmult_base(skey)
    rskey = randombytes(crypto_scalarmult_SCALARBYTES)
    rpkey = crypto_scalarmult_base(rskey)
    sesskey = crypto_scalarmult(skey, rpkey)
    h = crypto_generichash(sesskey, crypto_generichash_BYTES_MIN)
    odata = b'123'
    nonce = randombytes(crypto_aead_chacha20poly1305_ietf_NPUBBYTES)
    cipherText = crypto_aead_chacha20poly1305_ietf_encrypt(
        odata, None, nonce, sesskey)
    plainText = crypto_aead_chacha20poly1305_ietf_decrypt(
        cipherText, None, nonce, sesskey)
    print(plainText)



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from typing import Optional
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QDialog, QWidget, QApplication
from LANDrop.ui_aboutdialog import Ui_AboutDialog


class AboutDialog(QDialog):
    def __init__(self, parent: Optional[QWidget] = None) -> None:
        super().__init__(parent)
        self.ui = Ui_AboutDialog()
        self.ui.setupUi(self)
        self.setWindowFlag(Qt.WindowStaysOnTopHint)
        self.ui.aboutText.setHtml(
            self.ui.aboutText.toHtml(
            ).replace(
                "%1", QApplication.applicationName()
            ).replace(
                "%2", QApplication.applicationVersion()
            ).replace(
                "%3", QApplication.organizationName()))
        self.ui.aboutQtButton.clicked.connect(self.aboutQtButtonClicked)

    def aboutQtButtonClicked(self) -> None:
        QApplication.aboutQt()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\asdfz\Downloads\PyLANDrop\aboutdialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.setWindowModality(QtCore.Qt.WindowModal)
        AboutDialog.resize(400, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.aboutText = QtWidgets.QTextBrowser(AboutDialog)
        self.aboutText.setObjectName("aboutText")
        self.verticalLayout.addWidget(self.aboutText)
        self.aboutQtButton = QtWidgets.QPushButton(AboutDialog)
        self.aboutQtButton.setObjectName("aboutQtButton")
        self.verticalLayout.addWidget(self.aboutQtButton)

        self.retranslateUi(AboutDialog)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "About"))
        self.aboutText.setHtml(_translate("AboutDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
                                          "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
                                          "p, li { white-space: pre-wrap; }\n"
                                          "</style></head><body style=\" font-family:\'Sans Serif\'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:12pt; font-weight:600;\">%1 %2</span></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Drop any files to any devices on your LAN.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">BSD 3-Clause License</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Copyright (c) 2021, %3</p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">All rights reserved.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS &quot;AS IS&quot; AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">LANDrop Icon</span></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">This work is licensed under the Creative Commons Attribution-NonCommercial-NoDerivatives 4.0 International License. To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-nd/4.0/.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:12pt; font-weight:600;\">3rd Party Software &amp; Artwork</span></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">libsodium</span></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">ISC License</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Copyright (c) 2013-2021</p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Frank Denis &lt;j at pureftpd dot org&gt;</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission notice appear in all copies.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">THE SOFTWARE IS PROVIDED &quot;AS IS&quot; AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.</p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:10pt; font-weight:600;\">Material Design Icons</span></p>\n"
                                          "<p align=\"justify\" style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p align=\"justify\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at</p>\n"
                                          "<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">http://www.apache.org/licenses/LICENSE-2.0</p>\n"
                                          "<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p>\n"
                                          "<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.</p></body></html>"))
        self.aboutQtButton.setText(_translate("AboutDialog", "About Qt"))



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
from typing import Union

from PyQt5.QtCore import QObject, QByteArray
from LANDrop.sodium import *


class Crypto:
    _inited: bool = False

    @property
    def inited(self) -> bool:
        return Crypto._inited

    @inited.setter
    def inited(self, value: bool) -> None:
        Crypto._inited = value

    def __init__(self) -> None:
        self.sessionKey: bytes = b""
        self.init()
        self.secretKey = randombytes(crypto_scalarmult_SCALARBYTES)
        self.publicKey = crypto_scalarmult_base(self.secretKey)

    @classmethod
    def init(cls) -> None:
        if cls._inited:
            return
        # if sodium_init() == -1:
        #     raise RuntimeError(QObject().tr("Unable to initialize libsodium."))
        cls._inited = True

    def publicKeySize(self) -> int:
        return crypto_aead_chacha20poly1305_ietf_KEYBYTES

    def localPublicKey(self) -> bytes:
        return self.publicKey

    def setRemotePublicKey(self, remotePublicKey: Union[QByteArray, bytes]) -> None:
        remotePublicKey = bytes(remotePublicKey)
        try:
            self.sessionKey = crypto_scalarmult(
                self.secretKey, remotePublicKey)
        except RuntimeError:
            raise RuntimeError(QObject().tr(
                "Unable to calculate session key."))

    def sessionKeyDigest(self) -> str:
        h = crypto_generichash(self.sessionKey, crypto_generichash_BYTES_MIN)
        hash_ = 0
        for i in range(8):
            hash_ |= h[i] << (i * 8)
        return f"{hash_ % 1000000:0>6}"

    def encrypt(self, data: Union[QByteArray, bytes]) -> bytes:
        data = bytes(data)
        nonce = randombytes(crypto_aead_chacha20poly1305_ietf_NPUBBYTES)
        cipherText = crypto_aead_chacha20poly1305_ietf_encrypt(
            data, None, nonce, self.sessionKey)
        return nonce + cipherText

    def decrypt(self, data: Union[QByteArray, bytes]) -> bytes:
        data = bytes(data)
        if len(data) < crypto_aead_chacha20poly1305_ietf_NPUBBYTES:
            raise RuntimeError(QObject().tr("Cipher text too short."))
        nonce = data[:crypto_aead_chacha20poly1305_ietf_NPUBBYTES]
        cipherText = data[crypto_aead_chacha20poly1305_ietf_NPUBBYTES:]
        try:
            plainText = crypto_aead_chacha20poly1305_ietf_decrypt(
                cipherText, None, nonce, self.sessionKey)
        except (TypeError, ValueError) as e:
            raise RuntimeError(QObject().tr(
                "Decryption failed.") + " Error: " + str(e))
        return plainText



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from PyQt5.QtCore import QFile, QIODevice, QStringListModel, QFileInfo, Qt
from PyQt5.QtWidgets import QDialog, QWidget, QDialogButtonBox, QMessageBox, QApplication, QFileDialog
from PyQt5.QtGui import QDropEvent, QDragEnterEvent
from LANDrop.ui_selectfilesdialog import Ui_SelectFilesDialog
from LANDrop.discoveryservice import DiscoveryService
from LANDrop.sendtodialog import SendToDialog
from typing import List


class SelectFilesDialog(QDialog):

    def __init__(self, parent: QWidget, discoveryService: DiscoveryService) -> None:
        super().__init__(parent)
        self.ui = Ui_SelectFilesDialog()
        self.discoveryService = discoveryService
        self.files: List[QFile] = []
        self.filesStringListModel = QStringListModel()
        self._d = None

        self.ui.setupUi(self)
        self.setWindowFlag(Qt.WindowStaysOnTopHint)
        self.ui.addButton.clicked.connect(self.addButtonClicked)
        self.ui.removeButton.clicked.connect(self.removeButtonClicked)
        self.ui.filesListView.setModel(self.filesStringListModel)

        self.ui.buttonBox.button(QDialogButtonBox.Ok).setText(self.tr("Send"))
        self.ui.buttonBox.button(
            QDialogButtonBox.Cancel).setText(self.tr("Cancel"))

    def addFile(self, filename: str) -> None:
        for file in self.files:
            if file.fileName() == filename:
                return

        fp = QFile(filename)
        if not fp.open(QIODevice.ReadOnly):
            QMessageBox.critical(self, QApplication.applicationName(),
                                 self.tr("Unable to open file %1. Skipping.")
                                 .replace("%1", filename))
            return

        if fp.isSequential():
            QMessageBox.critical(self, QApplication.applicationName(),
                                 self.tr("%1 is not a regular file. Skipping.")
                                 .replace("%1", filename))
            return

        self.files.append(fp)

    def updateFileStringListModel(self) -> None:
        l = []
        for file in self.files:
            l.append(QFileInfo(file).fileName())

        self.filesStringListModel.setStringList(l)

    def addButtonClicked(self) -> None:
        filenames, _ = QFileDialog.getOpenFileNames(
            None, self.tr("Select File(s) to be Sent"))
        if not filenames:
            return

        for filename in filenames:
            self.addFile(filename)

        self.updateFileStringListModel()

    def removeButtonClicked(self) -> None:

        indexes = self.ui.filesListView.selectionModel().selectedIndexes()
        removeList = []
        for i in indexes:
            removeList.append(self.files[i.row()])
        for fp in removeList:
            self.files.remove(fp)

        self.updateFileStringListModel()

    def accept(self) -> None:

        if not self.files:
            QMessageBox.warning(
                self, QApplication.applicationName(), self.tr("No file to be sent."))
            return

        self._d = SendToDialog(None, self.files, self.discoveryService)
        self._d.setAttribute(Qt.WA_DeleteOnClose)
        self._d.show()

        self.done(self.Accepted)

    def dragEnterEvent(self, event: QDragEnterEvent) -> None:
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dropEvent(self, event: QDropEvent) -> None:
        if event.mimeData().hasUrls():
            for url in event.mimeData().urls():
                self.addFile(url.toLocalFile())

            self.updateFileStringListModel()
            event.acceptProposedAction()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\asdfz\Downloads\PyLANDrop\filetransferdialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FileTransferDialog(object):
    def setupUi(self, FileTransferDialog):
        FileTransferDialog.setObjectName("FileTransferDialog")
        FileTransferDialog.resize(400, 66)
        self.verticalLayout = QtWidgets.QVBoxLayout(FileTransferDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.statusLabel = QtWidgets.QLabel(FileTransferDialog)
        self.statusLabel.setText("")
        self.statusLabel.setObjectName("statusLabel")
        self.verticalLayout.addWidget(self.statusLabel)
        self.progressBar = QtWidgets.QProgressBar(FileTransferDialog)
        self.progressBar.setMaximum(10000)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout.addWidget(self.progressBar)

        self.retranslateUi(FileTransferDialog)
        QtCore.QMetaObject.connectSlotsByName(FileTransferDialog)

    def retranslateUi(self, FileTransferDialog):
        _translate = QtCore.QCoreApplication.translate
        FileTransferDialog.setWindowTitle(_translate("FileTransferDialog", "Transferring"))



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from base64 import encode
import json
from typing import Optional
from PyQt5.QtCore import QDir, QFileInfo, QFile, QIODevice, QObject, QUrl, QTimer
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtNetwork import QTcpSocket
from LANDrop.filetransfersession import FileTransferSession, State
from LANDrop.settings import Settings


class FileTransferReceiver(FileTransferSession):

    def __init__(self, parent: Optional['QObject'], socket: QTcpSocket) -> None:
        super().__init__(parent, socket)
        self.writingFile = None
        self.downloadPath = Settings.downloadPath()

    def respond(self, accepted: bool) -> None:
        obj = {
            "response": int(accepted)
        }
        self.encryptAndSend(json.dumps(
            obj, ensure_ascii=False), encode("utf-8"))

        if accepted:
            if not QDir().mkpath(self.downloadPath):
                self.errorOccurred.emit(
                    self.tr("Cannot create download path: ") + self.downloadPath)
                return

            if not QFileInfo(self.downloadPath).isWritable():
                self.errorOccurred.emit(
                    self.tr("Download path is not writable: ") + self.downloadPath)
                return

            self.state = State.TRANSFERRING
            self.createNextFile()
        else:
            self.socket.bytesWritten.connect(self.ended)

    def processReceivedData(self, data: bytes) -> None:
        if self.state == State.HANDSHAKE2:
            try:
                obj = json.loads(data)
            except json.JSONDecodeError:
                self.ended.emit()
                return

            deviceName = obj["device_name"]
            if not isinstance(deviceName, str):
                self.ended.emit()
                return

            filesJson = obj["files"]
            if not isinstance(filesJson, list):
                self.ended.emit()
                return

            filesJsonArray = filesJson
            if not filesJsonArray:
                self.ended.emit()
                return

            for v in filesJsonArray:
                if not isinstance(v, dict):
                    self.ended.emit()
                    return
                o = v

                filename = o["filename"]
                if not isinstance(filename, str):
                    self.ended.emit()
                    return

                size = o["size"]
                if not isinstance(size, (int, float)):
                    self.ended.emit()
                    return

                sizeInt = int(size)
                self.totalSize += sizeInt
                self.transferQ.append(FileTransferSession.FileMetadata(
                    filename.toString(), sizeInt))

            self.fileMetadataReady.emit(self.transferQ, self.totalSize, deviceName.toString(),
                                        self.crypto.sessionKeyDigest())
        elif self.state == State.TRANSFERRING:
            self.transferredSize += len(data)
            self.updateProgress.emit(
                float(self.transferredSize) / self.totalSize)
            tmpData = data
            while len(tmpData) > 0:
                curFile = self.transferQ[0]
                writeSize = min(curFile.size, len(tmpData))
                written = self.writingFile.write(tmpData[:writeSize])
                curFile.size -= written
                tmpData = tmpData[written:]
                if curFile.size == 0:
                    self.transferQ = self.transferQ[1:]
                    self.createNextFile()

    def createNextFile(self) -> None:
        while self.transferQ:
            curFile = self.transferQ[0]
            filename = self.downloadPath + QDir.separator() + curFile.filename
            if self.writingFile:
                self.writingFile.deleteLater()
                self.writingFile = None
            self.writingFile = QFile(filename, self)
            if not self.writingFile.open(QIODevice.WriteOnly):
                self.errorOccurred.emit(
                    self.tr("Unable to open file %1.").replace("%1", filename))
                return

            if curFile.size > 0:
                self.printMessage.emit(
                    self.tr("Receiving file %1...").replace("%1", curFile.filename))
                break

            self.transferQ = self.transferQ[1:]

        if not self.transferQ:
            if self.writingFile:
                self.writingFile.deleteLater()
                self.writingFile = None

            self.state = State.FINISHED
            QDesktopServices.openUrl(QUrl.fromLocalFile(self.downloadPath))
            self.printMessage.emit(self.tr("Done!"))
            self.socket.disconnectFromHost()
            QTimer.singleShot(5000, self.ended)

    def handshake1Finished(self):
        pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\asdfz\Downloads\PyLANDrop\selectfilesdialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SelectFilesDialog(object):
    def setupUi(self, SelectFilesDialog):
        SelectFilesDialog.setObjectName("SelectFilesDialog")
        SelectFilesDialog.resize(390, 383)
        SelectFilesDialog.setAcceptDrops(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(SelectFilesDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.instructionLabel = QtWidgets.QLabel(SelectFilesDialog)
        self.instructionLabel.setObjectName("instructionLabel")
        self.verticalLayout.addWidget(self.instructionLabel)
        self.filesListView = QtWidgets.QListView(SelectFilesDialog)
        self.filesListView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.filesListView.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.filesListView.setObjectName("filesListView")
        self.verticalLayout.addWidget(self.filesListView)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.addButton = QtWidgets.QPushButton(SelectFilesDialog)
        self.addButton.setObjectName("addButton")
        self.horizontalLayout.addWidget(self.addButton)
        self.removeButton = QtWidgets.QPushButton(SelectFilesDialog)
        self.removeButton.setObjectName("removeButton")
        self.horizontalLayout.addWidget(self.removeButton)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(SelectFilesDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SelectFilesDialog)
        self.buttonBox.accepted.connect(SelectFilesDialog.accept)
        self.buttonBox.rejected.connect(SelectFilesDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SelectFilesDialog)

    def retranslateUi(self, SelectFilesDialog):
        _translate = QtCore.QCoreApplication.translate
        SelectFilesDialog.setWindowTitle(_translate("SelectFilesDialog", "Select File(s) to be Sent"))
        self.instructionLabel.setText(_translate("SelectFilesDialog", "You can drag files to this window:"))
        self.addButton.setText(_translate("SelectFilesDialog", "Add..."))
        self.removeButton.setText(_translate("SelectFilesDialog", "Remove"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\asdfz\Downloads\PyLANDrop\sendtodialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SendToDialog(object):
    def setupUi(self, SendToDialog):
        SendToDialog.setObjectName("SendToDialog")
        SendToDialog.resize(218, 300)
        self.verticalLayout = QtWidgets.QVBoxLayout(SendToDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.addrLineEdit = QtWidgets.QLineEdit(SendToDialog)
        self.addrLineEdit.setText("")
        self.addrLineEdit.setObjectName("addrLineEdit")
        self.horizontalLayout.addWidget(self.addrLineEdit)
        self.colon = QtWidgets.QLabel(SendToDialog)
        self.colon.setObjectName("colon")
        self.horizontalLayout.addWidget(self.colon)
        self.portLineEdit = QtWidgets.QLineEdit(SendToDialog)
        self.portLineEdit.setText("")
        self.portLineEdit.setMaxLength(5)
        self.portLineEdit.setObjectName("portLineEdit")
        self.horizontalLayout.addWidget(self.portLineEdit)
        self.horizontalLayout.setStretch(0, 3)
        self.horizontalLayout.setStretch(2, 1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.hostsListView = QtWidgets.QListView(SendToDialog)
        self.hostsListView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.hostsListView.setObjectName("hostsListView")
        self.verticalLayout.addWidget(self.hostsListView)
        self.buttonBox = QtWidgets.QDialogButtonBox(SendToDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SendToDialog)
        self.buttonBox.accepted.connect(SendToDialog.accept)
        self.buttonBox.rejected.connect(SendToDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SendToDialog)

    def retranslateUi(self, SendToDialog):
        _translate = QtCore.QCoreApplication.translate
        SendToDialog.setWindowTitle(_translate("SendToDialog", "Send to"))
        self.addrLineEdit.setPlaceholderText(_translate("SendToDialog", "Address"))
        self.colon.setText(_translate("SendToDialog", ":"))
        self.portLineEdit.setPlaceholderText(_translate("SendToDialog", "Port"))



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import json
from typing import List, Optional
from PyQt5.QtCore import QObject, pyqtSignal, QSysInfo
from PyQt5.QtWidgets import QMessageBox, QApplication
from PyQt5.QtNetwork import QHostAddress, QUdpSocket, QNetworkInterface
from LANDrop.settings import Settings

DISCOVERY_PORT = 52637


class DiscoveryService(QObject):
    newHost = pyqtSignal(str, QHostAddress, int)  # deviceName,addr,port

    def __init__(self, parent: Optional['QObject'] = None) -> None:
        super().__init__(parent)
        self.socket = QUdpSocket()
        self.serverPort = None
        self.socket.readyRead.connect(self.socketReadyRead)

    def start(self, serverPort: int) -> None:
        self.serverPort = serverPort
        if not self.socket.bind(QHostAddress.Any, DISCOVERY_PORT):
            QMessageBox.warning(None, QApplication.applicationName(),
                                self.tr(
                                    "Unable to bind to port %1.\nYour device won't be discoverable."
            ).replace(
                                    "%1", str(DISCOVERY_PORT)))
        for addr in self.broadcastAddresses():
            self.sendInfo(addr, DISCOVERY_PORT)

    def refresh(self) -> None:
        obj = {"request": True}
        jsond = json.dumps(obj, ensure_ascii=False).encode('utf-8')
        for addr in self.broadcastAddresses():
            self.socket.writeDatagram(jsond, addr, DISCOVERY_PORT)

    def sendInfo(self, addr: QHostAddress, port: int) -> None:
        obj = {
            "request": False,
            "device_name": Settings.deviceName(),
            "device_type": QSysInfo.productType(),
            "port": self.serverPort if Settings.discoverable() else 0
        }
        self.socket.writeDatagram(json.dumps(
            obj, ensure_ascii=False).encode('utf-8'), addr, port)

    def isLocalAddress(self, addr: QHostAddress) -> bool:
        for address in QNetworkInterface.allAddresses():
            if addr.isEqual(address):
                return True
        return False

    def broadcastAddresses(self) -> List[QHostAddress]:
        ret = [QHostAddress.Broadcast]
        for i in QNetworkInterface.allInterfaces():
            if i.flags() & QNetworkInterface.CanBroadcast:
                for e in i.addressEntries():
                    ret.append(e.broadcast())

        return ret

    def socketReadyRead(self) -> None:
        while self.socket.hasPendingDatagrams():
            size = self.socket.pendingDatagramSize()
            data, addr, port = self.socket.readDatagram(size)

            if self.isLocalAddress(addr):
                continue
            try:
                obj = json.loads(data)
            except json.JSONDecodeError:
                continue
            request = obj["request"]
            if not isinstance(request, bool):
                continue
            if request:
                self.sendInfo(addr, port)
                continue
            deviceName = obj["device_name"]
            remotePort = obj["port"]
            if not isinstance(deviceName, str) or not isinstance(remotePort, (int, float)):
                continue
            self.newHost.emit(deviceName, addr, remotePort)



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
import json
from typing import Optional, List

from PyQt5.QtCore import QSysInfo, QFileInfo, QTimer, QObject, QFile
from PyQt5.QtNetwork import QTcpSocket

from LANDrop.filetransfersession import FileTransferSession, State
from LANDrop.settings import Settings

TRANSFER_QUANTA = 64000


class FileTransferSender(FileTransferSession):

    def __init__(self, parent: Optional[QObject], socket: QTcpSocket, files: List[QFile]) -> None:
        super().__init__(parent, socket)
        self.files = files
        self.socket.bytesWritten.connect(self.socketBytesWritten)

        for file in self.files:
            filename = QFileInfo(file).fileName()
            size = file.size()
            self.totalSize += size
            self.transferQ.append(
                FileTransferSession.FileMetadata(filename, size))

    def handshake1Finished(self) -> None:
        jsonFiles = []
        for metadata in self.transferQ:
            jsonFile = {"filename": metadata.filename, "size": metadata.size}
            jsonFiles.append(jsonFile)

        obj = {"device_name": Settings.deviceName(
        ), "device_type": QSysInfo.productType(), "files": jsonFiles}
        self.encryptAndSend(json.dumps(
            obj, ensure_ascii=False).encode("utf-8"))

    def processReceivedData(self, data: bytes) -> None:
        if self.state == State.HANDSHAKE2:
            try:
                obj = json.loads(data)
            except:
                self.errorOccurred.emit(self.tr("Handshake failed."))
                return

            response = obj["response"]
            if not isinstance(response, (float, int)):
                self.errorOccurred.emit(self.tr("Handshake failed."))
                return

            if int(response) == 0:
                self.errorOccurred.emit(
                    self.tr("The receiving device rejected your file(s)."))
                return
            self.state = State.TRANSFERRING
            self.socketBytesWritten()

    def socketBytesWritten(self) -> None:
        if self.state != State.TRANSFERRING or self.socket.bytesToWrite() > 0:
            return

        while self.transferQ:
            curFile = self.transferQ[0]
            if curFile.size == 0:
                self.transferQ = self.transferQ[1:]
                self.files = self.files[1:]
            else:
                self.printMessage.emit(
                    self.tr("Sending file %1...").replace("%1", curFile.filename))
                break

        if not self.transferQ:
            self.state = State.FINISHED
            self.printMessage.emit(self.tr("Done!"))
            self.socket.disconnectFromHost()
            QTimer.singleShot(5000, self.ended)
            return

        curFile = self.files[0]
        curMetadata = self.transferQ[0]
        data = curFile.read(TRANSFER_QUANTA)
        self.encryptAndSend(data)
        curMetadata.size -= len(data)
        self.transferredSize += len(data)
        self.updateProgress.emit(float(self.transferredSize) / self.totalSize)

    def respond(self, accepted: bool) -> None:
        raise RuntimeError("respond not implemented")



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, self
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    self list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    self software without specific prior written permission.
#
# self SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF self SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from typing import List, Optional
from PyQt5.QtCore import Qt, QLocale
from PyQt5.QtWidgets import QWidget, QDialog, QMessageBox, QApplication
from LANDrop.ui_filetransferdialog import Ui_FileTransferDialog
from LANDrop.filetransfersession import FileTransferSession


class FileTransferDialog(QDialog):
    def __init__(self, parent: Optional[QWidget], session: FileTransferSession):
        super().__init__(parent)
        self.ui = Ui_FileTransferDialog()
        self.session = session
        self.errored = False
        self.questionBox = QMessageBox(self)

        self.ui.setupUi(self)
        self.setWindowFlag(Qt.WindowStaysOnTopHint)

        self.questionBox.setIcon(QMessageBox.Question)
        self.questionBox.setWindowTitle(QApplication.applicationName())
        self.questionBox.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        self.questionBox.setDefaultButton(QMessageBox.Yes)
        self.questionBox.finished.connect(self.respond)

        self.session.setParent(self)
        self.session.printMessage.connect(self.ui.statusLabel.setText)
        self.session.updateProgress.connect(self.sessionUpdateProgress)
        self.session.errorOccurred.connect(self.sessionErrorOccurred)
        self.session.fileMetadataReady.connect(self.sessionFileMetadataReady)
        self.session.ended.connect(self.accept)
        self.session.start()

    def respond(self, result: int) -> None:
        response = result == QMessageBox.Yes
        self.session.respond(response)
        if not response:
            self.hide()

    def sessionUpdateProgress(self, progress: float) -> None:
        self.ui.progressBar.setValue(self.ui.progressBar.maximum() * progress)

    def sessionErrorOccurred(self, msg: str) -> None:
        if self.errored:
            return
        self.errored = True
        if self.isVisible():
            QMessageBox.critical(self, QApplication.applicationName(), msg)
        self.done(self.Rejected)

    def sessionFileMetadataReady(self, metadata: List[FileTransferSession.FileMetadata], totalSize: int,
                                 deviceName: str, sessionKeyDigest: str) -> None:
        self.show()

        totalSizeStr: str = self.locale().formattedDataSize(
            totalSize, 2, QLocale.DataSizeTraditionalFormat)

        if len(metadata) == 1:
            msg = self.tr("%1 would like to share a file \"%2\" of size %3.").replace("%1",
                                                                                      deviceName).replace("%2",
                                                                                                          metadata[
                                                                                                              0].filename).replace(
                "%3", totalSizeStr)
        else:
            msg = self.tr("%1 would like to share %2 files of total size %3."
                          ).replace("%1", deviceName
                                    ).replace("%2", str(len(metadata))).replace(
                "%3", totalSizeStr)

        msg += self.tr("\nConfirm that the code \"%1\" is shown on the sending device.").replace(
            "%1", sessionKeyDigest)
        msg += self.tr("\nWould you like to receive it?")

        self.questionBox.setText(msg)
        self.questionBox.show()



# BSD 3-Clause License
#
# Copyright (c) 2021, LANDrop
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, self
#    list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    self list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
#    contributors may be used to endorse or promote products derived from
#    self software without specific prior written permission.
#
# self SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF self SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# /

# include <QMessageBox>
# include <QPushButton>

# include "filetransferdialog.h"
# include "filetransfersender.h"
# include "sendtodialog.h"
# include "ui_sendtodialog.h"

from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtNetwork import *
from typing import List
from LANDrop.ui_sendtodialog import Ui_SendToDialog
from dataclasses import dataclass
from LANDrop.discoveryservice import DiscoveryService
from LANDrop.filetransfersender import FileTransferSender
from LANDrop.filetransferdialog import FileTransferDialog


class SendToDialog(QDialog):
    @dataclass
    class Endpoint:
        addr: QHostAddress
        port: int

    def __init__(self, parent: QWidget, files: List[QFile], discoveryService: DiscoveryService) -> None:
        super().__init__(parent)
        self.ui = Ui_SendToDialog()
        self.files = files
        self.hostsStringListModel = QStringListModel()
        self.endpoints: List[SendToDialog.Endpoint] = []
        self.discoveryTimer = QTimer()
        self.socket = QTcpSocket()
        self.socketTimeoutTimer = QTimer()

        self.ui.setupUi(self)
        self.setWindowFlag(Qt.WindowStaysOnTopHint)
        self.ui.hostsListView.setModel(self.hostsStringListModel)
        self.ui.hostsListView.clicked.connect(self.hostsListViewClicked)
        self.ui.hostsListView.doubleClicked.connect(self.ui.buttonBox.accepted)

        self.ui.buttonBox.button(QDialogButtonBox.Ok).setText(self.tr("Send"))
        self.ui.buttonBox.button(
            QDialogButtonBox.Cancel).setText(self.tr("Cancel"))

        discoveryService.newHost.connect(self.newHost)
        self.discoveryTimer.timeout.connect(discoveryService.refresh)
        self.discoveryTimer.start(1000)
        discoveryService.refresh()

        self.socketTimeoutTimer.timeout.connect(self.socketTimeout)
        self.socketTimeoutTimer.setSingleShot(True)

    def newHost(self, deviceName: str, addr: QHostAddress, port: int) -> None:

        str_list = self.hostsStringListModel.stringList()
        if port == 0:
            for i in range(len(self.endpoints)):
                if self.endpoints[i].addr.isEqual(addr):
                    del self.endpoints[i]
                    del str_list[i]
                    self.hostsStringListModel.setStringList(str_list)
                    return
            return

        for i in range(len(self.endpoints)):
            if self.endpoints[i].addr.isEqual(addr):
                if str_list[i] != deviceName:
                    str_list[i] = deviceName
                    self.hostsStringListModel.setStringList(str_list)

                self.endpoints[i].port = port
                return
        self.endpoints.append(self.Endpoint(addr, port))
        str_list.append(deviceName)
        self.hostsStringListModel.setStringList(str_list)

    def hostsListViewClicked(self, index: QModelIndex) -> None:

        endpoint: SendToDialog.Endpoint = self.endpoints[index.row()]
        isV4 = True  # FIXME: 可能出错
        ipv4 = endpoint.addr.toIPv4Address()
        addr = QHostAddress(ipv4).toString(
        ) if isV4 else endpoint.addr.toString()
        self.ui.addrLineEdit.setText(addr)
        self.ui.portLineEdit.setText(str(endpoint.port))

    def accept(self) -> None:

        addr = self.ui.addrLineEdit.text()
        ok = False
        try:
            port = int(self.ui.portLineEdit.text())
            if port < 0 or port > 65535:
                raise ValueError
            ok = True
        except ValueError:
            pass

        if not ok:
            QMessageBox.critical(self, QApplication.applicationName(),
                                 self.tr("Invalid port. Please enter a number between 1 and 65535."))
            return

        self.socket = QTcpSocket(self)
        self.socket.connected.connect(self.socketConnected)
        self.socket.error.connect(self.socketErrorOccurred)

        self.socket.connectToHost(addr, port)
        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
        self.setCursor(QCursor(Qt.WaitCursor))
        self.socketTimeoutTimer.start(5000)

    def socketConnected(self) -> None:
        self.socketTimeoutTimer.stop()
        sender = FileTransferSender(None, self.socket, self.files)
        self._d = FileTransferDialog(None, sender)
        self._d.setAttribute(Qt.WA_DeleteOnClose)
        self._d.show()
        self.done(self.Accepted)

    def socketErrorOccurred(self) -> None:
        self.socketTimeoutTimer.stop()
        self.socket.disconnectFromHost()
        self.socket.close()
        self.socket.deleteLater()
        QMessageBox.critical(
            self, QApplication.applicationName(), self.socket.errorString())
        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
        self.setCursor(QCursor(Qt.ArrowCursor))

    def socketTimeout(self) -> None:
        self.socket.disconnectFromHost()
        self.socket.close()
        self.socket.deleteLater()
        QMessageBox.critical(
            self, QApplication.applicationName(), self.tr("Connection timed out"))
        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
        self.setCursor(QCursor(Qt.ArrowCursor))



import setuptools

with open("requirements.txt") as fin:
    REQUIRED_PACKAGES = fin.read()

setuptools.setup(
    name='LANDrop',
    version='0.4.0a1',
    description='LANDrop is a cross-platform tool that you can use to conveniently transfer photos, videos, and other types of files to other devices on the same local network.',
    author='heiheiyoyo',
    author_email='543425864@qq.com',
    url='https://github.com/heiheiyoyo/PyLANDrop',
    install_requires=REQUIRED_PACKAGES,
    packages=['LANDrop'],
    license="BSD-3",
    entry_points={'console_scripts': ['landrop = LANDrop.__main__:main']}
)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\asdfz\Downloads\PyLANDrop\settingsdialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SettingsDialog(object):
    def setupUi(self, SettingsDialog):
        SettingsDialog.setObjectName("SettingsDialog")
        SettingsDialog.resize(400, 265)
        self.verticalLayout = QtWidgets.QVBoxLayout(SettingsDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.basicGroupBox = QtWidgets.QGroupBox(SettingsDialog)
        self.basicGroupBox.setObjectName("basicGroupBox")
        self.formLayout_3 = QtWidgets.QFormLayout(self.basicGroupBox)
        self.formLayout_3.setObjectName("formLayout_3")
        self.deviceNameLabel = QtWidgets.QLabel(self.basicGroupBox)
        self.deviceNameLabel.setObjectName("deviceNameLabel")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.deviceNameLabel)
        self.deviceNameLineEdit = QtWidgets.QLineEdit(self.basicGroupBox)
        self.deviceNameLineEdit.setMaxLength(64)
        self.deviceNameLineEdit.setObjectName("deviceNameLineEdit")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.deviceNameLineEdit)
        self.downloadPathLabel = QtWidgets.QLabel(self.basicGroupBox)
        self.downloadPathLabel.setObjectName("downloadPathLabel")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.downloadPathLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.downloadPathLineEdit = QtWidgets.QLineEdit(self.basicGroupBox)
        self.downloadPathLineEdit.setObjectName("downloadPathLineEdit")
        self.horizontalLayout.addWidget(self.downloadPathLineEdit)
        self.downloadPathSelectButton = QtWidgets.QToolButton(self.basicGroupBox)
        self.downloadPathSelectButton.setObjectName("downloadPathSelectButton")
        self.horizontalLayout.addWidget(self.downloadPathSelectButton)
        self.formLayout_3.setLayout(1, QtWidgets.QFormLayout.FieldRole, self.horizontalLayout)
        self.discoverableLabel = QtWidgets.QLabel(self.basicGroupBox)
        self.discoverableLabel.setObjectName("discoverableLabel")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.discoverableLabel)
        self.discoverableCheckBox = QtWidgets.QCheckBox(self.basicGroupBox)
        self.discoverableCheckBox.setText("")
        self.discoverableCheckBox.setObjectName("discoverableCheckBox")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.discoverableCheckBox)
        self.checkForUpdatesLabel = QtWidgets.QLabel(self.basicGroupBox)
        self.checkForUpdatesLabel.setObjectName("checkForUpdatesLabel")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.checkForUpdatesLabel)
        self.checkForUpdatesButton = QtWidgets.QPushButton(self.basicGroupBox)
        self.checkForUpdatesButton.setObjectName("checkForUpdatesButton")
        self.formLayout_3.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.checkForUpdatesButton)
        self.verticalLayout.addWidget(self.basicGroupBox)
        self.advancedGroupBox = QtWidgets.QGroupBox(SettingsDialog)
        self.advancedGroupBox.setObjectName("advancedGroupBox")
        self.formLayout = QtWidgets.QFormLayout(self.advancedGroupBox)
        self.formLayout.setObjectName("formLayout")
        self.serverPortLabel = QtWidgets.QLabel(self.advancedGroupBox)
        self.serverPortLabel.setObjectName("serverPortLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.serverPortLabel)
        self.serverPortLineEdit = QtWidgets.QLineEdit(self.advancedGroupBox)
        self.serverPortLineEdit.setMaxLength(5)
        self.serverPortLineEdit.setObjectName("serverPortLineEdit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.serverPortLineEdit)
        self.verticalLayout.addWidget(self.advancedGroupBox)
        self.buttonBox = QtWidgets.QDialogButtonBox(SettingsDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SettingsDialog)
        self.buttonBox.accepted.connect(SettingsDialog.accept)
        self.buttonBox.rejected.connect(SettingsDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialog)

    def retranslateUi(self, SettingsDialog):
        _translate = QtCore.QCoreApplication.translate
        SettingsDialog.setWindowTitle(_translate("SettingsDialog", "Settings"))
        self.basicGroupBox.setTitle(_translate("SettingsDialog", "Basic"))
        self.deviceNameLabel.setText(_translate("SettingsDialog", "Device Name"))
        self.downloadPathLabel.setText(_translate("SettingsDialog", "Download Path"))
        self.downloadPathSelectButton.setText(_translate("SettingsDialog", "..."))
        self.discoverableLabel.setText(_translate("SettingsDialog", "Discoverable"))
        self.checkForUpdatesLabel.setText(_translate("SettingsDialog", "Check for Updates"))
        self.checkForUpdatesButton.setText(_translate("SettingsDialog", "Check"))
        self.advancedGroupBox.setTitle(_translate("SettingsDialog", "Advanced"))
        self.serverPortLabel.setText(_translate("SettingsDialog", "Server Port"))
